import {
  Body,
  Controller,
  Get,
  Query,
  Put,
  Delete,
  Post,
  BadRequestException
} from '@nestjs/common';
import { ApiOperation, ApiTags } from '@nestjs/swagger';
import { Perm, definePermission } from '../auth/decorators/permission.decorator';
import { ApiSecurityAuth } from '~/common/decorators/swagger.decorator';
import { CompanyService } from './company.service';
import { ApiResult } from '~/common/decorators/api-result.decorator';
import { CompanyEntity } from './company.entity';
import { CompanyDto, CompanyQueryDto, CompanyUpdateDto } from './company.dto';
import { IdParam } from '~/common/decorators/id-param.decorator';
export const permissions = definePermission('app:company', {
  LIST: 'list',
  CREATE: 'create',
  READ: 'read',
  UPDATE: 'update',
  DELETE: 'delete'
} as const);

@ApiTags('Company - 公司') 
@ApiSecurityAuth()
@Controller('company')
export class CompanyController {
  constructor(private companyService: CompanyService) {}

  @Get()
  @ApiOperation({ summary: '获取公司列表' })
  @ApiResult({ type: [CompanyEntity], isPage: true })
  @Perm(permissions.LIST)
  async list(@Query() dto: CompanyQueryDto) {
    return this.companyService.findAll(dto);
  }

  @Get(':id')
  @ApiOperation({ summary: '获取公司信息' })
  @ApiResult({ type: CompanyDto })
  @Perm(permissions.READ)
  async info(@IdParam() id: number) {
    return this.companyService.info(id);
  }

  @Post()
  @ApiOperation({ summary: '新增公司' })
  @Perm(permissions.CREATE)
  async create(@Body() dto: CompanyDto): Promise<void> {
    await this.companyService.create(dto);
  }

  @Put(':id')
  @ApiOperation({ summary: '更新公司' })
  @Perm(permissions.UPDATE)
  async update(@IdParam() id: number, @Body() dto: CompanyUpdateDto): Promise<void> {
    await this.companyService.update(id, dto);
  }

  @Delete(':id')
  @ApiOperation({ summary: '删除公司' })
  @Perm(permissions.DELETE)
  async delete(@IdParam() id: number): Promise<void> {
    await this.companyService.delete(id);
  }

  @Put('unlink-attachments/:id')
  @ApiOperation({ summary: '附件解除关联' })
  @Perm(permissions.UPDATE)
  async unlinkAttachments(
    @IdParam() id: number,
    @Body() { fileIds }: CompanyUpdateDto
  ): Promise<void> {
    await this.companyService.unlinkAttachments(id, fileIds);
  }
}
