import { ApiProperty, IntersectionType, PartialType } from '@nestjs/swagger';
import { Transform } from 'class-transformer';
import {
  IsArray,
  IsDate,
  IsDateString,
  IsEnum,
  IsIn,
  IsInt,
  IsNumber,
  IsOptional,
  IsString,
  Matches,
  MinLength
} from 'class-validator';
import dayjs from 'dayjs';
import { PagerDto } from '~/common/dto/pager.dto';
import { MaterialsInOrOutEnum } from '~/constants/enum';
import { Storage } from '~/modules/tools/storage/storage.entity';
import { formatToDate } from '~/utils';

export class MaterialsInOutDto {
  @ApiProperty({ description: '产品' })
  @IsNumber()
  productId: number;

  @ApiProperty({ description: '单位(字典)' })
  @IsNumber()
  @IsOptional()
  unitId: number;

  @ApiProperty({ description: '入库或出库 0:入库 1:出库' })
  @IsEnum(MaterialsInOrOutEnum)
  inOrOut: MaterialsInOrOutEnum;

  @ApiProperty({ description: '时间' })
  @IsOptional()
  time: Date;

  @ApiProperty({ description: '数量' })
  @IsNumber()
  quantity: number;

  @ApiProperty({ description: '单价' })
  @IsNumber()
  unitPrice: number;

  @ApiProperty({ description: '金额' })
  @IsNumber()
  amount: number;

  @ApiProperty({ description: '经办人' })
  @IsOptional()
  @IsString()
  agent: string;

  @ApiProperty({ description: '领料单号' })
  @IsOptional()
  @IsString()
  issuanceNumber: string;

  @IsOptional()
  @IsString()
  @ApiProperty({ description: '备注' })
  remark: string;

  @IsOptional()
  @IsString()
  @ApiProperty({ description: '项目' })
  project: string;

  // @ApiProperty({ description: '附件' })
  // @IsOptional()
  // files: Storage[];
}

export class MaterialsInOutUpdateDto extends PartialType(MaterialsInOutDto) {
  @ApiProperty({ description: '附件' })
  @IsOptional()
  @IsArray()
  fileIds: number[];
}
export class MaterialsInOutQueryDto extends PagerDto<MaterialsInOutDto> {
  @ApiProperty({ description: '出入库时间YYYY-MM-DD' })
  @IsOptional()
  // @IsString()
  @Transform(params => {
    // 开始和结束时间用的是一天的开始和一天的结束的时分秒
    const date = params.value;
    return [
      date ? `${formatToDate(dayjs(date).startOf('month'))} 00:00:00` : null,
      date ? `${formatToDate(dayjs(date).endOf('month'))} 23:59:59` : null
    ];
  })
  time?: string[];

  @ApiProperty({ description: '入库或出库 0:入库 1:出库' })
  @IsOptional()
  @IsEnum(MaterialsInOrOutEnum)
  inOrOut: MaterialsInOrOutEnum;

  @ApiProperty({ description: '产品名称' })
  @IsOptional()
  @IsString()
  product: string;
}
