import { Injectable } from '@nestjs/common';
import { InjectEntityManager, InjectRepository } from '@nestjs/typeorm';

import { EntityManager, Repository } from 'typeorm';
import { Pagination } from '~/helper/paginate/pagination';
import { BusinessException } from '~/common/exceptions/biz.exception';
import { ErrorEnum } from '~/constants/error-code.constant';
import { paginate } from '~/helper/paginate';
import { Storage } from '~/modules/tools/storage/storage.entity';
import {
  MaterialsInOutQueryDto,
  MaterialsInOutDto,
  MaterialsInOutUpdateDto
} from './materials_in_out.dto';
import { MaterialsInOutEntity } from './materials_in_out.entity';
import { fieldSearch } from '~/shared/database/field-search';

@Injectable()
export class MaterialsInOutService {
  constructor(
    @InjectEntityManager() private entityManager: EntityManager,
    @InjectRepository(MaterialsInOutEntity)
    private materialsInOutRepository: Repository<MaterialsInOutEntity>,
    @InjectRepository(Storage)
    private storageRepository: Repository<Storage>
  ) {}
  /**
   * 查询所有出入库记录
   */
  async findAll({
    page,
    pageSize,
    product: productName,
    ...ext
  }: MaterialsInOutQueryDto): Promise<Pagination<MaterialsInOutEntity>> {
    const sqb = this.materialsInOutRepository
      .createQueryBuilder('materialsInOut')
      .leftJoin('materialsInOut.files', 'files')
      .leftJoin('materialsInOut.product', 'product')
      .leftJoin('product.unit', 'unit')
      .addSelect(['files.id', 'files.path', 'product.id', 'product.name', 'unit.id', 'unit.label'])
      .where(fieldSearch(ext))
      .andWhere('materialsInOut.isDelete = 0')
      .addOrderBy('materialsInOut.time', 'DESC');
    if (productName) {
      sqb.andWhere('product.name like :productName', { productName: `%${productName}%` });
    }
    let pageData = await paginate<MaterialsInOutEntity>(sqb, {
      page,
      pageSize
    });
    // 产品表中的单位字段需要单独处理
    pageData.items = pageData.items.map(materialsInOut => {
      materialsInOut.unit = materialsInOut.product.unit;
      delete materialsInOut.product.unit;
      return materialsInOut;
    });
    return pageData;
  }

  /**
   * 新增
   */
  async create(dto: MaterialsInOutDto): Promise<void> {
    await this.materialsInOutRepository.insert(dto);
  }

  /**
   * 更新
   */
  async update(id: number, { fileIds, ...data }: Partial<MaterialsInOutUpdateDto>): Promise<void> {
    await this.entityManager.transaction(async manager => {
      await manager.update(MaterialsInOutEntity, id, {
        ...data
      });
      const materialsInOut = await this.materialsInOutRepository
        .createQueryBuilder('materialsInOut')
        .leftJoinAndSelect('materialsInOut.files', 'files')
        .where('materialsInOut.id = :id', { id })
        .getOne();
      if (fileIds?.length) {
        const count = await this.storageRepository
          .createQueryBuilder('storage')
          .where('storage.id in(:fileIds)', { fileIds })
          .getCount();
        if (count !== fileIds?.length) {
          throw new BusinessException(ErrorEnum.STORAGE_NOT_FOUND);
        }
        // 附件要批量更新
        await manager
          .createQueryBuilder()
          .relation(MaterialsInOutEntity, 'files')
          .of(id)
          .addAndRemove(fileIds, materialsInOut.files);
      }
    });
  }

  /**
   * 删除
   */
  async delete(id: number): Promise<void> {
    // 出入库比较重要，做逻辑删除
    await this.materialsInOutRepository.update(id, { isDelete: 1 });
  }

  /**
   * 获取单个出入库信息
   */
  async info(id: number) {
    const info = await this.materialsInOutRepository
      .createQueryBuilder('materialsInOut')
      .where({
        id
      })
      .andWhere('materialsInOut.isDelete = 0')
      .getOne();
    return info;
  }

  /**
   * 解除附件关联
   * @param id 出入库ID
   * @param fileIds 附件ID
   */
  async unlinkAttachments(id: number, fileIds: number[]) {
    await this.entityManager.transaction(async manager => {
      const materialsInOut = await this.materialsInOutRepository
        .createQueryBuilder('materialsInOut')
        .leftJoinAndSelect('materialsInOut.files', 'files')
        .where('materialsInOut.id = :id', { id })
        .getOne();
      const linkedFiles = materialsInOut.files
        .map(item => item.id)
        .filter(item => !fileIds.includes(item));
      // 附件要批量更新
      await manager
        .createQueryBuilder()
        .relation(MaterialsInOutEntity, 'files')
        .of(id)
        .addAndRemove(linkedFiles, materialsInOut.files);
    });
  }
}
