import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';

import { In, Like, Repository } from 'typeorm';

import { BusinessException } from '~/common/exceptions/biz.exception';
import { ErrorEnum } from '~/constants/error-code.constant';
import { paginate } from '~/helper/paginate';
import { Pagination } from '~/helper/paginate/pagination';
import { DictTypeEntity } from '~/modules/system/dict-type/dict-type.entity';

import { DictTypeDto, DictTypeQueryDto } from './dict-type.dto';
import { DictTypeStatusEnum } from '~/constants/enum';

@Injectable()
export class DictTypeService {
  constructor(
    @InjectRepository(DictTypeEntity)
    private dictTypeRepository: Repository<DictTypeEntity>
  ) {}

  /**
   * 罗列所有配置
   */
  async page({
    page,
    pageSize,
    name,
    code
  }: DictTypeQueryDto): Promise<Pagination<DictTypeEntity>> {
    const queryBuilder = this.dictTypeRepository.createQueryBuilder('dict_type').where({
      ...(name && { name: Like(`%${name}%`) }),
      ...(code && { code: Like(`%${code}%`) })
    });

    return paginate(queryBuilder, { page, pageSize });
  }

  /** 一次性获取所有的字典类型 */
  async getAll({ withItems, storeCodes }: DictTypeQueryDto) {
    const sqb = this.dictTypeRepository
      .createQueryBuilder('dict_type')
      .addSelect(['dict_type.name', 'dict_type.code', 'dict_type.status']);
    if (withItems) {
      sqb
        .leftJoin('dict_type.dictItems', 'dictItems')
        .addSelect(['dictItems.id', 'dictItems.value', 'dictItems.label', 'dictItems.status']);
    }
    sqb.where('dict_type.status =:status', { status: DictTypeStatusEnum.ENABLE });
    withItems && sqb.andWhere('dictItems.status =:status', { status: DictTypeStatusEnum.ENABLE });
    storeCodes && sqb.andWhere({ code: In(storeCodes) });
    return sqb.getMany();
  }

  /**
   * 获取参数总数
   */
  async countConfigList(): Promise<number> {
    return this.dictTypeRepository.count();
  }

  /**
   * 新增
   */
  async create(dto: DictTypeDto): Promise<void> {
    await this.dictTypeRepository.insert(dto);
  }

  /**
   * 更新
   */
  async update(id: number, dto: Partial<DictTypeDto>): Promise<void> {
    await this.dictTypeRepository.update(id, dto);
  }

  /**
   * 删除
   */
  async delete(id: number): Promise<void> {
    await this.dictTypeRepository.delete(id);
  }

  /**
   * 查询单个
   */
  async findOne(id: number): Promise<DictTypeEntity> {
    return this.dictTypeRepository.findOneBy({ id });
  }

  async isExistKey(name: string): Promise<void | never> {
    const result = await this.dictTypeRepository.findOneBy({ name });
    if (result) throw new BusinessException(ErrorEnum.DICT_NAME_EXISTS);
  }
}
